@@ -19,9 +19,6 @@
 #
 
 import io
-import logging
-
-logger = logging.getLogger(__name__)
 
 
 class PyatsNotInstalled(Exception):
@@ -33,20 +30,19 @@ class PyatsDeviceNotFound(Exception):
 
 
 class ClPyats:
-    """Creates a pyATS object that can be used to run commands
-    against a device either in exec mode ``show version`` or in
-    configuration mode ``interface gi0/0\\\\nno shut``.
-
-    :param lab: The lab which should be used with pyATS
-    :type lab: models.Lab
-    :param hostname: Force hostname/ip and port for console terminal server
-    :type hostname: str
-    :raises PyatsNotInstalled: when pyATS can not be found
-    :raises PyatsDeviceNotFound: when the device can not be found
-    """
-
     def __init__(self, lab, hostname=None):
-        """Constructor method"""
+        """
+        Creates a pyATS object that can be used to run commands
+        against a device either in exec mode ``show version`` or in
+        configuration mode ``interface gi0/0 \\n no shut``.
+
+        :param lab: The lab which should be used with pyATS
+        :type lab: models.Lab
+        :param hostname: Force hostname/ip and port for console terminal server
+        :type hostname: str
+        :raises PyatsNotInstalled: when pyATS can not be found
+        :raises PyatsDeviceNotFound: when the device can not be found
+        """
         self._pyats_installed = False
         self._lab = lab
         self._hostname = hostname
@@ -65,7 +61,8 @@ class ClPyats:
             raise PyatsNotInstalled
 
     def sync_testbed(self, username, password):
-        """Syncs the the testbed from the server. Note that this
+        """
+        Syncs the testbed from the server. Note that this
         will always fetch the latest topology data from the server.
 
         :param username: the username that will be inserted into
@@ -85,7 +82,8 @@ class ClPyats:
         self._testbed = data
 
     def run_command(self, node_label, command):
-        """Run a command on the device in exec mode
+        """
+        Run a command on the device in `exec` mode.
 
         :param node_label: the label / title of the device
         :type node_label: str
@@ -108,8 +106,9 @@ class ClPyats:
         return pyats_device.execute(command, log_stdout=False)
 
     def run_config_command(self, node_label, command):
-        """Run a command on the device in configure mode. pyATS
-        handles the change into configure mode automatically.
+        """
+        Run a command on the device in `configure` mode. pyATS
+        handles the change into `configure` mode automatically.
 
         :param node_label: the label / title of the device
         :type node_label: str
@@ -132,6 +131,6 @@ class ClPyats:
         return pyats_device.configure(command, log_stdout=False)
 
     def cleanup(self):
-        """cleans up the pyATS connections"""
+        """Cleans up the pyATS connections."""
         for pyats_device in self._connections:
             pyats_device.destroy()
