@@ -19,29 +19,29 @@
 #
 
 import logging
+import warnings
 from functools import total_ordering
 
-logger = logging.getLogger(__name__)
+_LOGGER = logging.getLogger(__name__)
 
 
 @total_ordering
 class Interface:
-    """A VIRL2 network interface, part of a node.
-
-    :param iid: interface ID
-    :type iid: str
-    :param node: node object
-    :type node: models.Node
-    :param label: the label of the interface
-    :type label: str
-    :param slot: the slot of the interface
-    :type slot: int
-    :param iface_type: the type of the interface, defaults to "physical"
-    :type iface_type: str, optional
-    """
-
     def __init__(self, iid, node, label, slot, iface_type="physical"):
-        """Constructor method"""
+        """
+        A VIRL2 network interface, part of a node.
+
+        :param iid: interface ID
+        :type iid: str
+        :param node: node object
+        :type node: models.Node
+        :param label: the label of the interface
+        :type label: str
+        :param slot: the slot of the interface
+        :type slot: int
+        :param iface_type: the type of the interface, defaults to "physical"
+        :type iface_type: str
+        """
         self.id = iid
         self.node = node
         self.type = iface_type
@@ -57,10 +57,6 @@ class Interface:
         }
         self.ip_snooped_info = {"mac_address": None, "ipv4": None, "ipv6": None}
 
-    @property
-    def is_physical(self):
-        return self.type == "physical"
-
     def __eq__(self, other):
         if not isinstance(other, Interface):
             return False
@@ -95,17 +91,46 @@ class Interface:
     def _base_url(self):
         return self.lab_base_url + "/interfaces/{}".format(self.id)
 
-    def links(self):
+    @property
+    def physical(self):
+        """Whether the interface is physical."""
         self.node.lab.sync_topology_if_outdated()
-        return [lnk for lnk in self.node.lab.links() if self in lnk.interfaces]
+        return self.type == "physical"
 
-    def degree(self):
-        self.node.lab.sync_topology_if_outdated()
-        return len(self.links())
+    @property
+    def connected(self):
+        """Whether the interface is connected to a link."""
+        return self.link is not None
 
-    def is_connected(self):
+    @property
+    def state(self):
+        self.node.lab.sync_states_if_outdated()
+        return self._state
+
+    @property
+    def link(self):
+        """Is link if connected, otherwise None."""
         self.node.lab.sync_topology_if_outdated()
-        return self.degree() > 0
+        for link in self.node.lab.links():
+            if self in link.interfaces:
+                return link
+
+    @property
+    def peer_interface(self):
+        link = self.link
+        if link is None:
+            return None
+        interfaces = link.interfaces
+        if interfaces[0] is self:
+            return interfaces[1]
+        return interfaces[0]
+
+    @property
+    def peer_node(self):
+        peer_interface = self.peer_interface
+        if peer_interface is None:
+            return None
+        return peer_interface.node
 
     @property
     def readbytes(self):
@@ -142,31 +167,34 @@ class Interface:
         self.node.lab.sync_l3_addresses_if_outdated()
         return self.ip_snooped_info["ipv6"]
 
-    def peer_interfaces(self):
-        self.node.lab.sync_topology_if_outdated()
-        ifaces = set()
-        for link in self.links():
-            if link.interface_a.id == self.id:
-                ifaces.add(link.interface_b)
-            else:
-                ifaces.add(link.interface_a)
-        return ifaces
-
-    def peer_nodes(self):
-        self.node.lab.sync_topology_if_outdated()
-        return {iface.node for iface in self.peer_interfaces()}
-
-    def remove_on_server(self):
-        logger.info("Removing interface %s", self)
-
-        url = self._base_url
-        response = self.node.session.delete(url)
-        response.raise_for_status()
+    @property
+    def is_physical(self):
+        warnings.warn("Deprecated, use .physical instead.", DeprecationWarning)
+        return self.physical
 
     def as_dict(self):
         # TODO what should be here in 'data' key?
         return {"id": self.id, "node": self.node.id, "data": self.id}
 
+    def get_link_to(self, other_interface):
+        """
+        Returns the link between this interface and another.
+
+        :param other_interface: the other interface
+        :type other_interface: models.Interface
+        :returns: A Link
+        :rtype: models.Link
+        """
+        link = self.link
+        return link if other_interface in link.interfaces else None
+
+    def remove_on_server(self):
+        _LOGGER.info("Removing interface %s", self)
+
+        url = self._base_url
+        response = self.node.session.delete(url)
+        response.raise_for_status()
+
     def bring_up(self):
         url = self._base_url + "/state/start"
         response = self.session.put(url)
@@ -177,7 +205,25 @@ class Interface:
         response = self.session.put(url)
         response.raise_for_status()
 
-    @property
-    def state(self):
-        self.node.lab.sync_states_if_outdated()
-        return self._state
+    def peer_interfaces(self):
+        warnings.warn("Deprecated, use .peer_interface instead.", DeprecationWarning)
+        return {self.peer_interface}
+
+    def peer_nodes(self):
+        warnings.warn("Deprecated, use .peer_node instead.", DeprecationWarning)
+        return {self.peer_node}
+
+    def links(self):
+        warnings.warn("Deprecated, use .link instead.", DeprecationWarning)
+        link = self.link
+        if link is None:
+            return []
+        return [link]
+
+    def degree(self):
+        warnings.warn("Deprecated, use .connected instead.", DeprecationWarning)
+        return int(self.connected)
+
+    def is_connected(self):
+        warnings.warn("Deprecated, use .connected instead.", DeprecationWarning)
+        return self.connected
