@@ -21,47 +21,14 @@
 import logging
 import time
 from functools import total_ordering
-from itertools import chain
 
 from ..exceptions import InterfaceNotFound
 
-logger = logging.getLogger(__name__)
-
-flatten = chain.from_iterable
+_LOGGER = logging.getLogger(__name__)
 
 
 @total_ordering
 class Node:
-    """A VIRL2 Node object. Typically a virtual machine representing a router, switch or server.
-
-    :param lab: the Lab this nodes belongs to
-    :type lab: models.Lab
-    :param nid: the Node ID
-    :type nid: str
-    :param node_definition: The node definition of this node
-    :type node_definition: str
-    :param image_definition: The image definition of this node
-    :type image_definition: str
-    :param config: The day0 configuration of this node
-    :type config: str
-    :param x: X coordinate on topology canvas
-    :type x: int
-    :param y: Y coordinate on topology canvas
-    :type y: int
-    :param ram: memory of node in MiB (if applicable)
-    :type ram: int
-    :param cpus: Amount of CPUs in this node (if applicable)
-    :type cpus: int
-    :param cpu_limit: CPU limit (default at 100%)
-    :type cpu_limit: int
-    :param data_volume: Size in GiB of 2nd HDD (if > 0)
-    :type data_volume: int
-    :param boot_disk_size: Size in GiB of boot disk (will expand to this size)
-    :type boot_disk_size: int
-    :param tags: List of tags List[str, str]
-    :type tags: list
-    """
-
     def __init__(
         self,
         lab,
@@ -79,7 +46,37 @@ class Node:
         boot_disk_size,
         tags,
     ):
-        """Constructor method"""
+        """
+        A VIRL2 Node object. Typically a virtual machine representing a router,
+        switch or server.
+
+        :param lab: the Lab this nodes belongs to
+        :type lab: models.Lab
+        :param nid: the Node ID
+        :type nid: str
+        :param node_definition: The node definition of this node
+        :type node_definition: str
+        :param image_definition: The image definition of this node
+        :type image_definition: str
+        :param config: The day0 configuration of this node
+        :type config: str
+        :param x: X coordinate on topology canvas
+        :type x: int
+        :param y: Y coordinate on topology canvas
+        :type y: int
+        :param ram: memory of node in MiB (if applicable)
+        :type ram: int
+        :param cpus: Amount of CPUs in this node (if applicable)
+        :type cpus: int
+        :param cpu_limit: CPU limit (default at 100%)
+        :type cpu_limit: int
+        :param data_volume: Size in GiB of 2nd HDD (if > 0)
+        :type data_volume: int
+        :param boot_disk_size: Size in GiB of boot disk (will expand to this size)
+        :type boot_disk_size: int
+        :param tags: List of tags
+        :type tags: List[str, str]
+        """
         self.lab = lab
         self.id = nid
         self._label = label
@@ -96,6 +93,7 @@ class Node:
         self._data_volume = data_volume
         self._boot_disk_size = boot_disk_size
         self._tags = tags
+        self._compute_id = None
 
         self.statistics = {"cpu_usage": 0, "disk_read": 0, "disk_write": 0}
 
@@ -108,22 +106,25 @@ class Node:
         return "Node: {}".format(self._label)
 
     def __repr__(self):
-        return "{}({!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r})".format(
-            self.__class__.__name__,
-            str(self.lab),
-            self.id,
-            self._label,
-            self._node_definition,
-            self._image_definition,
-            self._config,
-            self._x,
-            self._y,
-            self._ram,
-            self._cpus,
-            self._cpu_limit,
-            self._data_volume,
-            self._boot_disk_size,
-            self._tags,
+        return (
+            "{}({!r}, {!r}, {!r}, {!r}, {!r}, {!r}, "
+            "{!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r}, {!r})".format(
+                self.__class__.__name__,
+                str(self.lab),
+                self.id,
+                self._label,
+                self._node_definition,
+                self._image_definition,
+                self._config,
+                self._x,
+                self._y,
+                self._ram,
+                self._cpus,
+                self._cpu_limit,
+                self._data_volume,
+                self._boot_disk_size,
+                self._tags,
+            )
         )
 
     def __eq__(self, other):
@@ -145,7 +146,7 @@ class Node:
 
     def physical_interfaces(self):
         self.lab.sync_topology_if_outdated()
-        return [iface for iface in self.interfaces() if iface.is_physical]
+        return [iface for iface in self.interfaces() if iface.physical]
 
     def create_interface(self, slot=None, wait=False):
         """
@@ -170,25 +171,30 @@ class Node:
         to connect to other nodes!
 
         :returns: an interface or None, if all existing ones are connected
-        :rtype: models.interface
+        :rtype: models.Interface
         """
         for iface in self.interfaces():
-            if not iface.is_connected() and iface.is_physical:
+            if not iface.connected and iface.physical:
                 return iface
         return None
 
     def peer_interfaces(self):
-        self.lab.sync_topology_if_outdated()
-        peer_ifaces = (iface.peer_interfaces() for iface in self.interfaces())
-        return list(set(flatten(peer_ifaces)))
+        peer_ifaces = {
+            iface.peer_interface for iface in self.interfaces() if iface is not None
+        }
+        return list(peer_ifaces)
 
     def peer_nodes(self):
-        self.lab.sync_topology_if_outdated()
-        return list(set(iface.node for iface in self.peer_interfaces()))
+        return list({iface.node for iface in self.peer_interfaces()})
 
     def links(self):
-        self.lab.sync_topology_if_outdated()
-        return list(set(flatten(iface.links() for iface in self.interfaces())))
+        """
+        :returns: list of links
+        :rtype: list[models.Link]
+        """
+        return list(
+            {link for iface in self.interfaces() if (link := iface.link) is not None}
+        )
 
     def degree(self):
         self.lab.sync_topology_if_outdated()
@@ -306,8 +312,13 @@ class Node:
         self._set_node_property("node_definition", value)
         self._image_definition = value
 
+    @property
+    def compute_id(self):
+        self.lab.sync_topology_if_outdated()
+        return self._compute_id
+
     def _set_node_property(self, key, val):
-        logger.info("Setting node property %s %s: %s", self, key, val)
+        _LOGGER.info("Setting node property %s %s: %s", self, key, val)
         node_url = "{}".format(self._base_url)
         response = self.session.patch(url=node_url, json={key: val})
         response.raise_for_status()
@@ -347,8 +358,36 @@ class Node:
                 return iface
         raise InterfaceNotFound("{}:{}".format(slot, self))
 
+    def get_links_to(self, other_node):
+        """
+        Returns all links between this node and another.
+
+        :param other_node: the other node
+        :type other_node: models.Node
+        :returns: a list of links
+        :rtype: list[models.Link]
+        """
+        links = []
+        for link in self.links():
+            if other_node in link.nodes:
+                links.append(link)
+        return links
+
+    def get_link_to(self, other_node):
+        """
+        Returns one link between this node and another.
+
+        :param other_node: the other node
+        :type other_node: models.Node
+        :returns: a link
+        :rtype: models.Link
+        """
+        for link in self.links():
+            if other_node in link.nodes:
+                return link
+
     def wait_until_converged(self, max_iterations=None, wait_time=None):
-        logger.info("Waiting for node %s to converge", self.id)
+        _LOGGER.info("Waiting for node %s to converge", self.id)
         max_iter = (
             self.lab.wait_max_iterations if max_iterations is None else max_iterations
         )
@@ -356,11 +395,11 @@ class Node:
         for index in range(max_iter):
             converged = self.has_converged()
             if converged:
-                logger.info("Node %s has converged", self.id)
+                _LOGGER.info("Node %s has converged", self.id)
                 return
 
             if index % 10 == 0:
-                logging.info(
+                _LOGGER.info(
                     "Node has not converged, attempt %s/%s, waiting...",
                     index,
                     max_iter,
@@ -371,7 +410,7 @@ class Node:
             self.id,
             max_iter,
         )
-        logger.error(msg)
+        _LOGGER.error(msg)
         # after maximum retries are exceeded and node has not converged
         # error must be raised - it makes no sense to just log info
         # and let client fail with something else if wait is explicitly
@@ -431,7 +470,7 @@ class Node:
         return response.json()
 
     def remove_on_server(self):
-        logger.info("Removing node %s", self)
+        _LOGGER.info("Removing node %s", self)
         url = self._base_url
         response = self.session.delete(url)
         response.raise_for_status()
@@ -479,7 +518,8 @@ class Node:
         to work, the device has to be attached to the external network
         in bridge mode and must run DHCP to acquire an IP address.
         """
-        # TODO: can optimise the sync of l3 to only be for the node rather than whole lab
+        # TODO: can optimise the sync of l3 to only be for the node
+        # rather than whole lab
         url = self._base_url + "/layer3_addresses"
         response = self.session.get(url)
         response.raise_for_status()
@@ -503,7 +543,6 @@ class Node:
 
     def update(self, node_data, exclude_configurations):
         if "data" in node_data:
-            # logger.warning("Deprecated since 2.4 (will be removed in 2.5)")
             node_data = node_data["data"]
         self._label = node_data["label"]
         self._x = node_data["x"]
@@ -516,6 +555,7 @@ class Node:
         self._data_volume = node_data["data_volume"]
         self._boot_disk_size = node_data["boot_disk_size"]
         self._tags = node_data["tags"]
+        self._compute_id = node_data.get("compute_id")
         if not exclude_configurations:
             self._config = node_data.get("configuration")
 
